include <threads.scad>
include <SG90.scad>

function inch(x) = x*25.4;

wall=2.4;

fan_w=40.2;
fan_h=20;
fan_screw_w=32;
fex=0.5;
fan_ex_lip=5.5;
fan_ex_bottom=6;
fan_ex_top=3;

pipe_od=inch(1.660); pipe_id=inch(1.380); pipe_exitd=fan_w+4*wall+fex; // 1-1/4"
//pipe_od=inch(1.315); pipe_id=inch(1.049); pipe_exitd=pipe_od+12; // 1"
pipe_h=inch(0.3); // no more than 1 inch
pipe_taper=inch(1/16); // 1/16 for NPT, 0 for conduit

servo_w=12.0;
servo_h=22.6;
servo_big_d=12.0;
servo_little_d=5.4;
servo_tip=4.9;

dial_od=(fan_w+4*wall+fex)*sqrt(2) - (3+2*wall);

fan_to_pipe=max(max(pipe_h+1, ((fan_w+2*wall+fex)*sqrt(2) - pipe_exitd)/2),
  pipe_h+(fan_w-2-pipe_id)/2+(pipe_od-pipe_taper-pipe_id)/3);
fan_to_dial=(dial_od-(fan_w+fex))/2;

e=0.01;

difference() {
  union() {
    mount_grill();
    translate([0,0,fan_to_pipe+fan_h+fan_ex_bottom + 
      servo_tip+servo_h+fan_ex_top+10]) {
      rotate([180])
        mount_servo();
        
      translate([0, 0, 10])
        servo_dial();
    }
  }
  *translate([0,0,-e]) cube([fan_w*2, fan_w*2, 200]);
}
//translate([-40,0,0]) cylinder(pipe_h, d=40);
//cylinder(pipe_h, d=35.4);

module mount_grill() {
  difference() {
    hull() {
      cylinder(pipe_h, d=pipe_exitd);
      translate([-(fan_w+4*wall+fex)/2, -(fan_w+4*wall+fex)/2, fan_to_pipe])
       rcube([fan_w+4*wall+fex, fan_w+4*wall+fex, fan_h+fan_ex_bottom], 3+2*wall, $fn=16);
    }
    
    // pipe_od thread
    *translate([0,0,-inch(1)+pipe_h-e])
      cylinder(inch(1)+2*e, d1=pipe_od, d2=pipe_od-pipe_taper);
    // The OD here should subtract a fraction of pipe_taper so it starts/ends 
    // the proper diameter, but that became too tight a fit
    translate([0,0,-e])
      metric_thread(diameter=pipe_od, pitch=25.4/11.5, 
        length=pipe_h+2*e, taper=pipe_taper/25.4, internal=true);
    
    // Fan hole (widening)
    translate([0, 0, pipe_h-e]) {
      z1 = (pipe_od-pipe_taper-pipe_id)/3; // yes/3, a 56 degree angle should be ok to print
      cylinder(z1+2*e, d1=pipe_od-pipe_taper, d2=pipe_id);
      translate([0,0,z1]) cylinder((fan_w-2-pipe_id)/2+2*e, d1=pipe_id, d2=fan_w-2);
    }
    
    // Fan hole (secure)
    translate([-fan_w/2, -fan_w/2, fan_to_pipe])
      rcube([fan_w, fan_w, fan_h], 2, $fn=12);
    // Fan hole (open)
    translate([-(fan_w+2*wall+fex)/2, -(fan_w+2*wall+fex)/2, fan_to_pipe+fan_h/2-e])
      rcube([fan_w+2*wall+fex, fan_w+2*wall+fex, fan_h/2+fan_ex_bottom+2*e], 3+wall, $fn=16);
    // pry open notches
    translate([-8/2, -(fan_w+6*wall+fex)/2, fan_to_pipe+fan_h+fan_ex_bottom-1])
      cube([8, fan_w+6*wall+fex, 1+e]);
    // screw holes
    for (z=[0,180])
      rotate(z) translate([fan_screw_w/2, fan_screw_w/2, fan_to_pipe-5], $fn=10)
        cylinder(5.5+e, d=3);
  }
  
  translate([0,0,fan_to_pipe]) %fan_mock();
}

module mount_servo() {
  //jack_din=8.5; jack_dout=9.5; jack_inset=1.5; jack_inset2=wall; // jack
  jack_din=10.5; jack_dout=inch(9/16); jack_inset=2; jack_inset2=wall/2; // grommet 1/4"(ID)x9/16"(OD)
  jack_project = dial_od/2;
  
  difference() {
    union() {
      hull() {
        cylinder(servo_tip, d=dial_od);
        translate([-(fan_w+4*wall+fex)/2, -(fan_w+4*wall+fex)/2, servo_tip+servo_h+fan_ex_top-0.5])
          rcube([fan_w+4*wall+fex, fan_w+4*wall+fex, 0.5], 3+2*wall, $fn=16);
        // Jack flat
        translate([-(jack_dout+wall)/2, 0, servo_tip+wall])
          cube([jack_dout+wall, jack_project, jack_dout+wall/2]);
      }
      // connecting lip
      translate([-(fan_w+2*wall+fex-0.05)/2, -(fan_w+2*wall+fex-0.05)/2, servo_tip+servo_h+fan_ex_top-e])
        rcube([fan_w+2*wall+fex-0.05, fan_w+2*wall+fex-0.05, fan_ex_lip+e], 3+wall, $fn=16);
    }

    // Intake holes
    intersection() {
      union() {
        translate([0,0,-e]) linear_extrude(servo_tip+2*e)
          pie_slice(servo_big_d/2+wall/2, dial_od/2-wall/2, -45+3, 45);
        translate([0,0,-e]) linear_extrude(servo_tip+2*e) 
          pie_slice(servo_big_d/2+wall/2, dial_od/2-wall/2, 135+3, 225);
      }
      // angle the edges inward toward 2*wall
      translate([0,0,-e]) cylinder(servo_tip+2*e, d1=dial_od-wall, d2=dial_od-2*wall);
    }

    // Bulge out to fan size
    translate([0, 0, servo_tip]) hull() {
      translate([0, 0, -0.5]) cylinder(0.5, d=dial_od-2*wall);
      translate([-(fan_w+fex)/2, -(fan_w+fex)/2, servo_h+fan_ex_top-0.5])
        rcube([fan_w+fex, fan_w+fex, 0.5], 3+wall, $fn=16);
    }
    translate([-(fan_w+fex)/2, -(fan_w+fex)/2, servo_tip+servo_h+fan_ex_top-e])
      rcube([fan_w+fex, fan_w+fex, fan_ex_lip+2*e], 3+wall, $fn=16);

    // Servo
    %translate([0,0,servo_tip-e]) rotate([180, 0, 180]) SG90();
    translate([0,0,-e]) cylinder(servo_tip+2*e, d=servo_big_d, $fn=24);
    translate([0,-servo_big_d/2,-e]) cylinder(servo_tip+2*e, d=servo_little_d, $fn=12);
    //translate([-servo_w/2,-22,servo_tip+2.5]) cube([servo_w, 22, 4]);

    // jack hole
    translate([0,0,servo_tip+wall+jack_dout/2])
      rotate([90, 0, 180]) {
      cylinder(jack_project+e, d=jack_din, $fn=18);
      translate([0,0,jack_project-jack_inset])
        cylinder(jack_inset+e, d=jack_dout, $fn=18);
    }
    // Jack flat
    translate([-jack_dout/2, 0, servo_tip])
      cube([jack_dout, jack_project-jack_inset2-wall, jack_dout+2*wall]);
  }
}

module servo_arm(h) {
  cylinder(h, d=8);
  translate([0,0,2]) hull() {
    translate([0,0,0]) cylinder(h, d=6);
    translate([0,29/2,0]) cylinder(h, d=4.2, $fn=8); 
    translate([0,-29/2,0]) cylinder(h, d=4.2, $fn=8); 
  }
}

module servo_dial() {
  difference() {
    cylinder(3.5, d=dial_od);
    
    // Intake holes
    translate([0,0,-e]) linear_extrude(servo_tip+2*e)
      pie_slice(servo_big_d/2+wall/2, dial_od/2-wall/2, -45+3, 45);
    translate([0,0,-e]) linear_extrude(servo_tip+2*e) 
      pie_slice(servo_big_d/2+wall/2, dial_od/2-wall/2, 135+3, 225);
    
    translate([0,0,-e]) servo_arm(4+2*e);
  }
}

module fan_mock() {
  difference() {
    translate([-fan_w/2, -fan_w/2, 0])
      rcube([fan_w, fan_w, fan_h], 3.5);
     translate([0,0,-e]) cylinder(fan_h+2*e, d=fan_w-2.4);
  }
}

module pie_slice(r1, r2, start_angle, end_angle) {
    R = r2 * sqrt(2) + 1;
    a0 = (4 * start_angle + 0 * end_angle) / 4;
    a1 = (3 * start_angle + 1 * end_angle) / 4;
    a2 = (2 * start_angle + 2 * end_angle) / 4;
    a3 = (1 * start_angle + 3 * end_angle) / 4;
    a4 = (0 * start_angle + 4 * end_angle) / 4;
    if(end_angle > start_angle)
        intersection() {
        difference() {
          circle(r2);
          circle(r1, $fn=24);
        }
        polygon([
            [0,0],
            [R * cos(a0), R * sin(a0)],
            [R * cos(a1), R * sin(a1)],
            [R * cos(a2), R * sin(a2)],
            [R * cos(a3), R * sin(a3)],
            [R * cos(a4), R * sin(a4)],
            [0,0]
       ]);
    }
}

module rcube(size, radius, center=false)
{
  x = size[0];
  y = size[1];
  z = size[2];

  if (center)
  {
    linear_extrude(height=z)
      hull()
      {
        // place 4 circles in the corners, with the given radius
        translate([(-x/2)+radius, (-y/2)+radius, 0])
          circle(r=radius);

        translate([(x/2)-radius, (-y/2)+radius, 0])
          circle(r=radius);

        translate([(-x/2)+radius, (y/2)-radius, 0])
          circle(r=radius);

        translate([(x/2)-radius, (y/2)-radius, 0])
          circle(r=radius);
     }
  }
  else
  {
    linear_extrude(height=z)
      hull()
      {
        // place 4 circles in the corners, with the given radius
        translate([radius, radius, 0])
          circle(r=radius);

        translate([x - radius, radius, 0])
          circle(r=radius);

        translate([x - radius, y - radius, 0])
          circle(r=radius);

        translate([radius, y - radius, 0])
          circle(r=radius);
      }
  }
}
